# $Id: ZoomCalculator.pm,v 1.1 2002/06/29 20:25:09 joern Exp $

#-----------------------------------------------------------------------
# Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
# 
# This module is part of Video::DVDRip, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Video::DVDRip::GUI::ZoomCalculator;

use base Video::DVDRip::GUI::Window;

use strict;
use Carp;

sub multi_instance_window { 1 }

sub gtk_widgets			{ shift->{gtk_widgets}			}
sub set_gtk_widgets		{ shift->{gtk_widgets}		= $_[1] }

sub calc_results		{ shift->{calc_results}			}
sub set_calc_results		{ shift->{calc_results}		= $_[1] }

# GUI Stuff ----------------------------------------------------------

sub build {
	my $self = shift; $self->trace_in;

	$self->set_gtk_widgets ({});

	# build window -----------------------------------------------
	my $win = Gtk::Window->new ( -toplevel );
	$win->set_title($self->config('program_name'). " Zoom Calculator");
	$win->border_width(0);
	$win->realize;
	$win->set_default_size ( 570, 540 );

	# Register component and window ------------------------------
	$self->set_widget($win);
	$self->set_gtk_window_widget($win);

	# Build dialog -----------------------------------------------
	my $dialog_vbox = Gtk::VBox->new;
	$dialog_vbox->show;
	$dialog_vbox->set_border_width(10);
	$win->add($dialog_vbox);

	my ($frame, $vbox, $hbox, $button, $clist, $sw);

	# Project Queue ----------------------------------------------
	$frame = Gtk::Frame->new ("Zoom Calculations");
	$frame->show;
	$dialog_vbox->pack_start($frame, 1, 1, 0);
	$vbox = Gtk::VBox->new;
	$vbox->set_border_width(5);
	$vbox->show;
	$frame->add ($vbox);
	$sw = new Gtk::ScrolledWindow( undef, undef );
	$vbox->pack_start ($sw, 1, 1, 0);
	$sw->show;
	$sw->set_policy( 'automatic', 'automatic' );

	$clist = Gtk::CList->new_with_titles (
		"Aligned Result Size ",
		"BPKP    ",
		"Eff. AR    ",
		"AR Error    ",
		"Exact Result Size",
		"Clip2 (t/b/l/r)   ",
	);
	$clist->show,
	$sw->add ($clist);
	$clist->set_selection_mode( 'browse' ); 

	$self->gtk_widgets->{calc_clist} = $clist;

	$hbox = Gtk::HBox->new (1, 10);
	$hbox->show;
	$vbox->pack_start ($hbox, 0, 1, 0);
	
	$button = Gtk::Button->new_with_label (" Cancel ");
	$button->show;
	$button->signal_connect ("clicked", sub { $win->destroy } );
	$hbox->pack_start ($button, 0, 1, 0);

	$button = Gtk::Button->new_with_label (" Apply ");
	$button->show;
	$button->signal_connect ("clicked", sub { $self->apply_values } );
	$hbox->pack_start ($button, 0, 1, 0);

	$button = Gtk::Button->new_with_label (" Ok ");
	$button->show;
	$button->signal_connect ("clicked", sub {
		$self->apply_values;
		$win->destroy;
	} );
	$hbox->pack_start ($button, 0, 1, 0);

	$self->init_calc_list;

	$win->show;

	return 1;
}

sub init_calc_list {
	my $self = shift;
	
	my ($clist, $rows);

	$clist = $self->gtk_widgets->{calc_clist};

	$clist->thaw;
	$clist->clear;

	my $font = Gtk::Gdk::Font->load (
		"-*-helvetica-bold-r-*-*-*-120-*-*-*-*-*-*"
	);
	my $highlighted = $clist->style->copy;
	$highlighted->font($font);
	
	my $perfect = $clist->style->copy;
	$perfect->font($font);
	$perfect->fg('normal',$self->gdk_color('ff0000'));
	$perfect->fg('selected',$self->gdk_color('ff9999'));

	$font = Gtk::Gdk::Font->load (
		"-*-helvetica-medium-r-*-*-*-120-*-*-*-*-*-*"
	);
	my $normal = $clist->style->copy;
	$normal->font($font);

	my $calc_lref = $self->comp('project')->selected_title->calculator;	
	
	my $i = 0;
	foreach my $result ( @{$calc_lref} ) {
		$clist->append (
			"$result->{clip2_width}x$result->{clip2_height}",
			sprintf("%.2f", $result->{bpkp}),
			sprintf("%.4f", $result->{eff_ar}),
			sprintf("%.4f%%", $result->{ar_err}),
			"$result->{exact_width}x$result->{exact_height}",
			"$result->{clip2_top} / $result->{clip2_bottom} / ".
			"$result->{clip2_left} / $result->{clip2_right}",
		);
		if ( $result->{ar_err} < 0.000001 ) {
			$clist->set_row_style($i, $perfect);
		} else {
			$clist->set_row_style(
				$i,
				$result->{ar_err} < 0.3 ? $highlighted:$normal
			);
		}
		++$i;
	}

	$clist->select_row ( 0, 0 );

	$self->set_calc_results ($calc_lref);

	1;
}

sub apply_values {
	my $self = shift;

	my ($row) = $self->gtk_widgets->{calc_clist}->selection;

	my $result  = $self->calc_results->[$row];
	my $project = $self->comp('project');
	my $title   = $project->selected_title;

	$title->set_tc_zoom_width   ( $result->{zoom_width}   );
	$title->set_tc_zoom_height  ( $result->{zoom_height}  );
	$title->set_tc_clip1_left   ( $result->{clip1_left}   );
	$title->set_tc_clip1_right  ( $result->{clip1_right}  );
	$title->set_tc_clip1_top    ( $result->{clip1_top}    );
	$title->set_tc_clip1_bottom ( $result->{clip1_bottom} );
	$title->set_tc_clip2_left   ( $result->{clip2_left}   );
	$title->set_tc_clip2_right  ( $result->{clip2_right}  );
	$title->set_tc_clip2_top    ( $result->{clip2_top}    );
	$title->set_tc_clip2_bottom ( $result->{clip2_bottom} );

	$project->make_previews;
	$project->init_adjust_values;
	
	1;
}

1;
